# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["CategoryResourceResponse"]


class CategoryResourceResponse(BaseModel):
    category: str

    resource: str

    start_timestamp: datetime

    input_price: Optional[float] = None

    max_input_units: Optional[int] = None

    max_output_units: Optional[int] = None

    output_price: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .total_cost_data import TotalCostData

__all__ = ["BudgetHistoryResponse", "BudgetHistory"]


class BudgetHistory(BaseModel):
    budget_name: Optional[str] = None

    base_cost_estimate: Optional[Literal["max"]] = None

    budget_id: Optional[str] = None

    budget_reset_timestamp: Optional[datetime] = None

    budget_response_type: Optional[Literal["block", "allow"]] = None

    budget_tags: Optional[List[str]] = None

    budget_type: Optional[Literal["conservative", "liberal"]] = None

    max: Optional[float] = None

    totals: Optional[TotalCostData] = None


class BudgetHistoryResponse(BaseModel):
    budget_history: BudgetHistory

    request_id: str

    message: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["IngestBulkParams", "Event"]


class IngestBulkParams(TypedDict, total=False):
    events: Required[Iterable[Event]]

class Event(TypedDict, total=False):
    category: Required[str]

    input: Required[int]

    output: Required[int]

    resource: Required[str]

    budget_ids: Optional[List[str]]

    event_timestamp: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    experience_instance_id: Optional[str]

    request_tags: Optional[List[str]]

    user_id: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .resource_create_params import ResourceCreateParams as ResourceCreateParams
from .resource_list_response import ResourceListResponse as ResourceListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.experiences import type_create_params, type_update_params
from ...types.experiences.experience_type import ExperienceType
from ...types.experiences.type_list_response import TypeListResponse

__all__ = ["TypesResource", "AsyncTypesResource"]


class TypesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TypesResourceWithRawResponse:
        return TypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TypesResourceWithStreamingResponse:
        return TypesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        description: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceType:
        """Adds a new experience type.

        Instances of this type can be created via
        /experiences/instance.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/experiences/types",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                type_create_params.TypeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceType,
        )

    def retrieve(
        self,
        experience_type_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceType:
        """
        Get details for a specific Experience Type

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_type_id:
            raise ValueError(f"Expected a non-empty value for `experience_type_id` but received {experience_type_id!r}")
        return self._get(
            f"/api/v1/experiences/types/{experience_type_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceType,
        )

    def update(
        self,
        experience_type_id: str,
        *,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        name: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceType:
        """
        Updates an existing experience type.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_type_id:
            raise ValueError(f"Expected a non-empty value for `experience_type_id` but received {experience_type_id!r}")
        return self._patch(
            f"/api/v1/experiences/types/{experience_type_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                type_update_params.TypeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceType,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TypeListResponse:
        """Get all Experience Types"""
        return self._get(
            "/api/v1/experiences/types",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TypeListResponse,
        )

    def delete(
        self,
        experience_type_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceType:
        """Deletes an experience type.

        This will remove visiblity of this type from all
        dashboard stats.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_type_id:
            raise ValueError(f"Expected a non-empty value for `experience_type_id` but received {experience_type_id!r}")
        return self._delete(
            f"/api/v1/experiences/types/{experience_type_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceType,
        )


class AsyncTypesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTypesResourceWithRawResponse:
        return AsyncTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTypesResourceWithStreamingResponse:
        return AsyncTypesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        description: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceType:
        """Adds a new experience type.

        Instances of this type can be created via
        /experiences/instance.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/experiences/types",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                type_create_params.TypeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceType,
        )

    async def retrieve(
        self,
        experience_type_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceType:
        """
        Get details for a specific Experience Type

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_type_id:
            raise ValueError(f"Expected a non-empty value for `experience_type_id` but received {experience_type_id!r}")
        return await self._get(
            f"/api/v1/experiences/types/{experience_type_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceType,
        )

    async def update(
        self,
        experience_type_id: str,
        *,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        name: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceType:
        """
        Updates an existing experience type.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_type_id:
            raise ValueError(f"Expected a non-empty value for `experience_type_id` but received {experience_type_id!r}")
        return await self._patch(
            f"/api/v1/experiences/types/{experience_type_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                type_update_params.TypeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceType,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TypeListResponse:
        """Get all Experience Types"""
        return await self._get(
            "/api/v1/experiences/types",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TypeListResponse,
        )

    async def delete(
        self,
        experience_type_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceType:
        """Deletes an experience type.

        This will remove visiblity of this type from all
        dashboard stats.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_type_id:
            raise ValueError(f"Expected a non-empty value for `experience_type_id` but received {experience_type_id!r}")
        return await self._delete(
            f"/api/v1/experiences/types/{experience_type_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceType,
        )


class TypesResourceWithRawResponse:
    def __init__(self, types: TypesResource) -> None:
        self._types = types

        self.create = to_raw_response_wrapper(
            types.create,
        )
        self.retrieve = to_raw_response_wrapper(
            types.retrieve,
        )
        self.update = to_raw_response_wrapper(
            types.update,
        )
        self.list = to_raw_response_wrapper(
            types.list,
        )
        self.delete = to_raw_response_wrapper(
            types.delete,
        )


class AsyncTypesResourceWithRawResponse:
    def __init__(self, types: AsyncTypesResource) -> None:
        self._types = types

        self.create = async_to_raw_response_wrapper(
            types.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            types.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            types.update,
        )
        self.list = async_to_raw_response_wrapper(
            types.list,
        )
        self.delete = async_to_raw_response_wrapper(
            types.delete,
        )


class TypesResourceWithStreamingResponse:
    def __init__(self, types: TypesResource) -> None:
        self._types = types

        self.create = to_streamed_response_wrapper(
            types.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            types.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            types.update,
        )
        self.list = to_streamed_response_wrapper(
            types.list,
        )
        self.delete = to_streamed_response_wrapper(
            types.delete,
        )


class AsyncTypesResourceWithStreamingResponse:
    def __init__(self, types: AsyncTypesResource) -> None:
        self._types = types

        self.create = async_to_streamed_response_wrapper(
            types.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            types.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            types.update,
        )
        self.list = async_to_streamed_response_wrapper(
            types.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            types.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .cost_data import CostData as CostData
from .cost_details import CostDetails as CostDetails
from .requests_data import RequestsData as RequestsData
from .budget_response import BudgetResponse as BudgetResponse
from .ingest_response import IngestResponse as IngestResponse
from .total_cost_data import TotalCostData as TotalCostData
from .default_response import DefaultResponse as DefaultResponse
from .category_response import CategoryResponse as CategoryResponse
from .paged_budget_list import PagedBudgetList as PagedBudgetList
from .budget_list_params import BudgetListParams as BudgetListParams
from .ingest_bulk_params import IngestBulkParams as IngestBulkParams
from .experience_instance import ExperienceInstance as ExperienceInstance
from .ingest_units_params import IngestUnitsParams as IngestUnitsParams
from .budget_create_params import BudgetCreateParams as BudgetCreateParams
from .budget_update_params import BudgetUpdateParams as BudgetUpdateParams
from .bulk_ingest_response import BulkIngestResponse as BulkIngestResponse
from .category_list_response import CategoryListResponse as CategoryListResponse
from .budget_history_response import BudgetHistoryResponse as BudgetHistoryResponse
from .category_delete_response import CategoryDeleteResponse as CategoryDeleteResponse
from .category_resource_response import CategoryResourceResponse as CategoryResourceResponse
from .category_list_resources_response import CategoryListResourcesResponse as CategoryListResourcesResponse
from .category_delete_resource_response import CategoryDeleteResourceResponse as CategoryDeleteResourceResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["RequestsData"]


class RequestsData(BaseModel):
    blocked: int

    error: int

    successful: int

    total: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .experience_type import ExperienceType

__all__ = ["TypeListResponse"]

TypeListResponse: TypeAlias = List[ExperienceType]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal

import httpx

from .tags import (
    TagsResource,
    AsyncTagsResource,
    TagsResourceWithRawResponse,
    AsyncTagsResourceWithRawResponse,
    TagsResourceWithStreamingResponse,
    AsyncTagsResourceWithStreamingResponse,
)
from ...types import budget_list_params, budget_create_params, budget_update_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.budget_response import BudgetResponse
from ...types.default_response import DefaultResponse
from ...types.paged_budget_list import PagedBudgetList
from ...types.budget_history_response import BudgetHistoryResponse

__all__ = ["BudgetsResource", "AsyncBudgetsResource"]


class BudgetsResource(SyncAPIResource):
    @cached_property
    def tags(self) -> TagsResource:
        return TagsResource(self._client)

    @cached_property
    def with_raw_response(self) -> BudgetsResourceWithRawResponse:
        return BudgetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BudgetsResourceWithStreamingResponse:
        return BudgetsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        budget_name: str,
        max: float,
        base_cost_estimate: Literal["max"] | NotGiven = NOT_GIVEN,
        billing_model_id: Optional[str] | NotGiven = NOT_GIVEN,
        budget_response_type: Literal["block", "allow"] | NotGiven = NOT_GIVEN,
        budget_tags: Optional[List[str]] | NotGiven = NOT_GIVEN,
        budget_type: Literal["conservative", "liberal"] | NotGiven = NOT_GIVEN,
        cost_basis: Literal["base", "billed"] | NotGiven = NOT_GIVEN,
        currency: Literal["usd"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BudgetResponse:
        """
        Create a Budget

        Args:
          budget_name (str): The name of the budget.
          max (float): The maximum budget amount.
          base_cost_estimate (Union[float, Literal['max']], optional): The base cost estimate. Defaults to 'max'.
          budget_response_type (Literal['block', 'allow'], optional): The budget response type. Defaults to 'block'.
          budget_tags (Union[List[str], None], optional): List of budget tags. Defaults to None.
          budget_type (Literal['conservative', 'liberal'], optional): The budget type. Defaults to 'conservative'.
          extra_headers (Dict[str, str], optional): Additional headers for the request. Defaults to None.
          extra_query (Dict[str, str], optional): Additional query parameters. Defaults to None.
          extra_body (Dict[str, Any], optional): Additional body parameters. Defaults to None.
          timeout (Union[float, None], optional): The timeout for the request in seconds. Defaults to None.
        """
        return self._post(
            "/api/v1/budgets",
            body=maybe_transform(
                {
                    "budget_name": budget_name,
                    "max": max,
                    "base_cost_estimate": base_cost_estimate,
                    "billing_model_id": billing_model_id,
                    "budget_response_type": budget_response_type,
                    "budget_tags": budget_tags,
                    "budget_type": budget_type,
                    "cost_basis": cost_basis,
                    "currency": currency,
                },
                budget_create_params.BudgetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BudgetResponse,
        )

    def retrieve(
        self,
        budget_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BudgetResponse:
        """
        Get a Budget's details

        Args:
          budget_id (str): The ID of the budget.
          extra_headers (Dict[str, str], optional): Additional headers for the request. Defaults to None.
          extra_query (Dict[str, str], optional): Additional query parameters. Defaults to None.
          extra_body (Dict[str, Any], optional): Additional body parameters. Defaults to None.
          timeout (Union[float, None], optional): The timeout for the request in seconds. Defaults to None.
        """
        if not budget_id:
            raise ValueError(f"Expected a non-empty value for `budget_id` but received {budget_id!r}")
        return self._get(
            f"/api/v1/budgets/{budget_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BudgetResponse,
        )

    def update(
        self,
        budget_id: str,
        *,
        budget_name: Optional[str] | NotGiven = NOT_GIVEN,
        max: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BudgetResponse:
        """
        Update a Budget

        Args:
          budget_id (str): The ID of the budget.
          budget_name (Union[str, optional]): The updated name of the budget. Defaults to None.
          max (Union[float, optional])): The maximum budget amount. Defaults to None.
          extra_headers (Dict[str, str], optional): Additional headers for the request. Defaults to None.
          extra_query (Dict[str, str], optional): Additional query parameters. Defaults to None.
          extra_body (Dict[str, Any], optional): Additional body parameters. Defaults to None.
          timeout (Union[float, None], optional): The timeout for the request in seconds. Defaults to None.
        """
        if not budget_id:
            raise ValueError(f"Expected a non-empty value for `budget_id` but received {budget_id!r}")
        return self._put(
            f"/api/v1/budgets/{budget_id}",
            body=maybe_transform(
                {
                    "budget_name": budget_name,
                    "max": max,
                },
                budget_update_params.BudgetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BudgetResponse,
        )

    def list(
        self,
        *,
        budget_name: str | NotGiven = NOT_GIVEN,
        page_number: int | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        sort_ascending: bool | NotGiven = NOT_GIVEN,
        sort_by: str | NotGiven = NOT_GIVEN,
        tags: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PagedBudgetList:
        """
        Get all Budgets

        Args:
          extra_headers (Dict[str, str], optional): Additional headers for the request. Defaults to None.
          extra_query (Dict[str, str], optional): Additional query parameters. Defaults to None.
          extra_body (Dict[str, Any], optional): Additional body parameters. Defaults to None.
          timeout (Union[float, None], optional): The timeout for the request in seconds. Defaults to None.
        """
        return self._get(
            "/api/v1/budgets",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "budget_name": budget_name,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort_ascending": sort_ascending,
                        "sort_by": sort_by,
                        "tags": tags,
                    },
                    budget_list_params.BudgetListParams,
                ),
            ),
            cast_to=PagedBudgetList,
        )

    def delete(
        self,
        budget_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DefaultResponse:
        """
        Delete a Budget

        Args:
          budget_id (str): The ID of the budget.
          extra_headers (Dict[str, str], optional): Additional headers for the request. Defaults to None.
          extra_query (Dict[str, str], optional): Additional query parameters. Defaults to None.
          extra_body (Dict[str, Any], optional): Additional body parameters. Defaults to None.
          timeout (Union[float, None], optional): The timeout for the request in seconds. Defaults to None.
        """
        if not budget_id:
            raise ValueError(f"Expected a non-empty value for `budget_id` but received {budget_id!r}")
        return self._delete(
            f"/api/v1/budgets/{budget_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DefaultResponse,
        )

    def reset(
        self,
        budget_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BudgetHistoryResponse:
        """
        Reset the Budget

        Args:
          budget_id (str): The ID of the budget.
          extra_headers (Dict[str, str], optional): Additional headers for the request. Defaults to None.
          extra_query (Dict[str, str], optional): Additional query parameters. Defaults to None.
          extra_body (Dict[str, Any], optional): Additional body parameters. Defaults to None.
          timeout (Union[float, None], optional): The timeout for the request in seconds. Defaults to None.
        """
        if not budget_id:
            raise ValueError(f"Expected a non-empty value for `budget_id` but received {budget_id!r}")
        return self._post(
            f"/api/v1/budgets/{budget_id}/reset",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BudgetHistoryResponse,
        )


class AsyncBudgetsResource(AsyncAPIResource):
    @cached_property
    def tags(self) -> AsyncTagsResource:
        return AsyncTagsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBudgetsResourceWithRawResponse:
        return AsyncBudgetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBudgetsResourceWithStreamingResponse:
        return AsyncBudgetsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        budget_name: str,
        max: float,
        base_cost_estimate: Literal["max"] | NotGiven = NOT_GIVEN,
        billing_model_id: Optional[str] | NotGiven = NOT_GIVEN,
        budget_response_type: Literal["block", "allow"] | NotGiven = NOT_GIVEN,
        budget_tags: Optional[List[str]] | NotGiven = NOT_GIVEN,
        budget_type: Literal["conservative", "liberal"] | NotGiven = NOT_GIVEN,
        cost_basis: Literal["base", "billed"] | NotGiven = NOT_GIVEN,
        currency: Literal["usd"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BudgetResponse:
        """
        Create a Budget

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/budgets",
            body=await async_maybe_transform(
                {
                    "budget_name": budget_name,
                    "max": max,
                    "base_cost_estimate": base_cost_estimate,
                    "billing_model_id": billing_model_id,
                    "budget_response_type": budget_response_type,
                    "budget_tags": budget_tags,
                    "budget_type": budget_type,
                    "cost_basis": cost_basis,
                    "currency": currency,
                },
                budget_create_params.BudgetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BudgetResponse,
        )

    async def retrieve(
        self,
        budget_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BudgetResponse:
        """
        Get a Budget's details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not budget_id:
            raise ValueError(f"Expected a non-empty value for `budget_id` but received {budget_id!r}")
        return await self._get(
            f"/api/v1/budgets/{budget_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BudgetResponse,
        )

    async def update(
        self,
        budget_id: str,
        *,
        budget_name: Optional[str] | NotGiven = NOT_GIVEN,
        max: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BudgetResponse:
        """
        Update a Budget

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not budget_id:
            raise ValueError(f"Expected a non-empty value for `budget_id` but received {budget_id!r}")
        return await self._put(
            f"/api/v1/budgets/{budget_id}",
            body=await async_maybe_transform(
                {
                    "budget_name": budget_name,
                    "max": max,
                },
                budget_update_params.BudgetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BudgetResponse,
        )

    async def list(
        self,
        *,
        budget_name: str | NotGiven = NOT_GIVEN,
        page_number: int | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        sort_ascending: bool | NotGiven = NOT_GIVEN,
        sort_by: str | NotGiven = NOT_GIVEN,
        tags: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PagedBudgetList:
        """
        Get all Budgets

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/v1/budgets",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "budget_name": budget_name,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort_ascending": sort_ascending,
                        "sort_by": sort_by,
                        "tags": tags,
                    },
                    budget_list_params.BudgetListParams,
                ),
            ),
            cast_to=PagedBudgetList,
        )

    async def delete(
        self,
        budget_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DefaultResponse:
        """
        Delete a Budget

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not budget_id:
            raise ValueError(f"Expected a non-empty value for `budget_id` but received {budget_id!r}")
        return await self._delete(
            f"/api/v1/budgets/{budget_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DefaultResponse,
        )

    async def reset(
        self,
        budget_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BudgetHistoryResponse:
        """
        Reset the Budget

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not budget_id:
            raise ValueError(f"Expected a non-empty value for `budget_id` but received {budget_id!r}")
        return await self._post(
            f"/api/v1/budgets/{budget_id}/reset",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BudgetHistoryResponse,
        )


class BudgetsResourceWithRawResponse:
    def __init__(self, budgets: BudgetsResource) -> None:
        self._budgets = budgets

        self.create = to_raw_response_wrapper(
            budgets.create,
        )
        self.retrieve = to_raw_response_wrapper(
            budgets.retrieve,
        )
        self.update = to_raw_response_wrapper(
            budgets.update,
        )
        self.list = to_raw_response_wrapper(
            budgets.list,
        )
        self.delete = to_raw_response_wrapper(
            budgets.delete,
        )
        self.reset = to_raw_response_wrapper(
            budgets.reset,
        )

    @cached_property
    def tags(self) -> TagsResourceWithRawResponse:
        return TagsResourceWithRawResponse(self._budgets.tags)


class AsyncBudgetsResourceWithRawResponse:
    def __init__(self, budgets: AsyncBudgetsResource) -> None:
        self._budgets = budgets

        self.create = async_to_raw_response_wrapper(
            budgets.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            budgets.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            budgets.update,
        )
        self.list = async_to_raw_response_wrapper(
            budgets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            budgets.delete,
        )
        self.reset = async_to_raw_response_wrapper(
            budgets.reset,
        )

    @cached_property
    def tags(self) -> AsyncTagsResourceWithRawResponse:
        return AsyncTagsResourceWithRawResponse(self._budgets.tags)


class BudgetsResourceWithStreamingResponse:
    def __init__(self, budgets: BudgetsResource) -> None:
        self._budgets = budgets

        self.create = to_streamed_response_wrapper(
            budgets.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            budgets.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            budgets.update,
        )
        self.list = to_streamed_response_wrapper(
            budgets.list,
        )
        self.delete = to_streamed_response_wrapper(
            budgets.delete,
        )
        self.reset = to_streamed_response_wrapper(
            budgets.reset,
        )

    @cached_property
    def tags(self) -> TagsResourceWithStreamingResponse:
        return TagsResourceWithStreamingResponse(self._budgets.tags)


class AsyncBudgetsResourceWithStreamingResponse:
    def __init__(self, budgets: AsyncBudgetsResource) -> None:
        self._budgets = budgets

        self.create = async_to_streamed_response_wrapper(
            budgets.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            budgets.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            budgets.update,
        )
        self.list = async_to_streamed_response_wrapper(
            budgets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            budgets.delete,
        )
        self.reset = async_to_streamed_response_wrapper(
            budgets.reset,
        )

    @cached_property
    def tags(self) -> AsyncTagsResourceWithStreamingResponse:
        return AsyncTagsResourceWithStreamingResponse(self._budgets.tags)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.



from .._models import BaseModel

__all__ = ["ExperienceInstance"]


class ExperienceInstance(BaseModel):
    experience_instance_id: str

    request_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.categories import resource_create_params
from ...types.category_resource_response import CategoryResourceResponse
from ...types.categories.resource_list_response import ResourceListResponse

__all__ = ["ResourcesResource", "AsyncResourcesResource"]


class ResourcesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ResourcesResourceWithRawResponse:
        return ResourcesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResourcesResourceWithStreamingResponse:
        return ResourcesResourceWithStreamingResponse(self)

    def create(
        self,
        resource: str,
        *,
        category: str,
        input_price: float | NotGiven = NOT_GIVEN,
        max_input_units: int | NotGiven = NOT_GIVEN,
        max_output_units: int | NotGiven = NOT_GIVEN,
        output_price: float | NotGiven = NOT_GIVEN,
        start_timestamp: Union[str, datetime, None] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CategoryResourceResponse:
        """
        Create a Resource

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category:
            raise ValueError(f"Expected a non-empty value for `category` but received {category!r}")
        if not resource:
            raise ValueError(f"Expected a non-empty value for `resource` but received {resource!r}")
        return self._post(
            f"/api/v1/categories/{category}/resources/{resource}",
            body=maybe_transform(
                {
                    "input_price": input_price,
                    "max_input_units": max_input_units,
                    "max_output_units": max_output_units,
                    "output_price": output_price,
                    "start_timestamp": start_timestamp,
                },
                resource_create_params.ResourceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CategoryResourceResponse,
        )

    def retrieve(
        self,
        start_timestamp: Union[str, datetime],
        *,
        category: str,
        resource: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CategoryResourceResponse:
        """
        Get a Resource's version details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category:
            raise ValueError(f"Expected a non-empty value for `category` but received {category!r}")
        if not resource:
            raise ValueError(f"Expected a non-empty value for `resource` but received {resource!r}")
        if not start_timestamp:
            raise ValueError(f"Expected a non-empty value for `start_timestamp` but received {start_timestamp!r}")
        return self._get(
            f"/api/v1/categories/{category}/resources/{resource}/{start_timestamp}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CategoryResourceResponse,
        )

    def list(
        self,
        resource: str,
        *,
        category: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ResourceListResponse:
        """
        Get a list of versions of a Resource

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category:
            raise ValueError(f"Expected a non-empty value for `category` but received {category!r}")
        if not resource:
            raise ValueError(f"Expected a non-empty value for `resource` but received {resource!r}")
        return self._get(
            f"/api/v1/categories/{category}/resources/{resource}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ResourceListResponse,
        )

    def delete(
        self,
        start_timestamp: Union[str, datetime],
        *,
        category: str,
        resource: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CategoryResourceResponse:
        """
        Delete a version of the Resource

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category:
            raise ValueError(f"Expected a non-empty value for `category` but received {category!r}")
        if not resource:
            raise ValueError(f"Expected a non-empty value for `resource` but received {resource!r}")
        if not start_timestamp:
            raise ValueError(f"Expected a non-empty value for `start_timestamp` but received {start_timestamp!r}")
        return self._delete(
            f"/api/v1/categories/{category}/resources/{resource}/{start_timestamp}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CategoryResourceResponse,
        )


class AsyncResourcesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncResourcesResourceWithRawResponse:
        return AsyncResourcesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResourcesResourceWithStreamingResponse:
        return AsyncResourcesResourceWithStreamingResponse(self)

    async def create(
        self,
        resource: str,
        *,
        category: str,
        input_price: float | NotGiven = NOT_GIVEN,
        max_input_units: int | NotGiven = NOT_GIVEN,
        max_output_units: int | NotGiven = NOT_GIVEN,
        output_price: float | NotGiven = NOT_GIVEN,
        start_timestamp: Union[str, datetime, None] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CategoryResourceResponse:
        """
        Create a Resource

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category:
            raise ValueError(f"Expected a non-empty value for `category` but received {category!r}")
        if not resource:
            raise ValueError(f"Expected a non-empty value for `resource` but received {resource!r}")
        return await self._post(
            f"/api/v1/categories/{category}/resources/{resource}",
            body=await async_maybe_transform(
                {
                    "input_price": input_price,
                    "max_input_units": max_input_units,
                    "max_output_units": max_output_units,
                    "output_price": output_price,
                    "start_timestamp": start_timestamp,
                },
                resource_create_params.ResourceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CategoryResourceResponse,
        )

    async def retrieve(
        self,
        start_timestamp: Union[str, datetime],
        *,
        category: str,
        resource: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CategoryResourceResponse:
        """
        Get a Resource's version details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category:
            raise ValueError(f"Expected a non-empty value for `category` but received {category!r}")
        if not resource:
            raise ValueError(f"Expected a non-empty value for `resource` but received {resource!r}")
        if not start_timestamp:
            raise ValueError(f"Expected a non-empty value for `start_timestamp` but received {start_timestamp!r}")
        return await self._get(
            f"/api/v1/categories/{category}/resources/{resource}/{start_timestamp}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CategoryResourceResponse,
        )

    async def list(
        self,
        resource: str,
        *,
        category: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ResourceListResponse:
        """
        Get a list of versions of a Resource

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category:
            raise ValueError(f"Expected a non-empty value for `category` but received {category!r}")
        if not resource:
            raise ValueError(f"Expected a non-empty value for `resource` but received {resource!r}")
        return await self._get(
            f"/api/v1/categories/{category}/resources/{resource}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ResourceListResponse,
        )

    async def delete(
        self,
        start_timestamp: Union[str, datetime],
        *,
        category: str,
        resource: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CategoryResourceResponse:
        """
        Delete a version of the Resource

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category:
            raise ValueError(f"Expected a non-empty value for `category` but received {category!r}")
        if not resource:
            raise ValueError(f"Expected a non-empty value for `resource` but received {resource!r}")
        if not start_timestamp:
            raise ValueError(f"Expected a non-empty value for `start_timestamp` but received {start_timestamp!r}")
        return await self._delete(
            f"/api/v1/categories/{category}/resources/{resource}/{start_timestamp}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CategoryResourceResponse,
        )


class ResourcesResourceWithRawResponse:
    def __init__(self, resources: ResourcesResource) -> None:
        self._resources = resources

        self.create = to_raw_response_wrapper(
            resources.create,
        )
        self.retrieve = to_raw_response_wrapper(
            resources.retrieve,
        )
        self.list = to_raw_response_wrapper(
            resources.list,
        )
        self.delete = to_raw_response_wrapper(
            resources.delete,
        )


class AsyncResourcesResourceWithRawResponse:
    def __init__(self, resources: AsyncResourcesResource) -> None:
        self._resources = resources

        self.create = async_to_raw_response_wrapper(
            resources.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            resources.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            resources.list,
        )
        self.delete = async_to_raw_response_wrapper(
            resources.delete,
        )


class ResourcesResourceWithStreamingResponse:
    def __init__(self, resources: ResourcesResource) -> None:
        self._resources = resources

        self.create = to_streamed_response_wrapper(
            resources.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            resources.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            resources.list,
        )
        self.delete = to_streamed_response_wrapper(
            resources.delete,
        )


class AsyncResourcesResourceWithStreamingResponse:
    def __init__(self, resources: AsyncResourcesResource) -> None:
        self._resources = resources

        self.create = async_to_streamed_response_wrapper(
            resources.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            resources.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            resources.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            resources.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .budget_tags import BudgetTags

__all__ = ["TagRemoveResponse"]

TagRemoveResponse: TypeAlias = List[BudgetTags]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.



from ..._models import BaseModel

__all__ = ["ExperienceType"]


class ExperienceType(BaseModel):
    description: str

    experience_type_id: str

    name: str

    request_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .ingest import (
    IngestResource,
    AsyncIngestResource,
    IngestResourceWithRawResponse,
    AsyncIngestResourceWithRawResponse,
    IngestResourceWithStreamingResponse,
    AsyncIngestResourceWithStreamingResponse,
)
from .budgets import (
    BudgetsResource,
    AsyncBudgetsResource,
    BudgetsResourceWithRawResponse,
    AsyncBudgetsResourceWithRawResponse,
    BudgetsResourceWithStreamingResponse,
    AsyncBudgetsResourceWithStreamingResponse,
)
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)
from .experiences import (
    ExperiencesResource,
    AsyncExperiencesResource,
    ExperiencesResourceWithRawResponse,
    AsyncExperiencesResourceWithRawResponse,
    ExperiencesResourceWithStreamingResponse,
    AsyncExperiencesResourceWithStreamingResponse,
)

__all__ = [
    "BudgetsResource",
    "AsyncBudgetsResource",
    "BudgetsResourceWithRawResponse",
    "AsyncBudgetsResourceWithRawResponse",
    "BudgetsResourceWithStreamingResponse",
    "AsyncBudgetsResourceWithStreamingResponse",
    "IngestResource",
    "AsyncIngestResource",
    "IngestResourceWithRawResponse",
    "AsyncIngestResourceWithRawResponse",
    "IngestResourceWithStreamingResponse",
    "AsyncIngestResourceWithStreamingResponse",
    "CategoriesResource",
    "AsyncCategoriesResource",
    "CategoriesResourceWithRawResponse",
    "AsyncCategoriesResourceWithRawResponse",
    "CategoriesResourceWithStreamingResponse",
    "AsyncCategoriesResourceWithStreamingResponse",
    "ExperiencesResource",
    "AsyncExperiencesResource",
    "ExperiencesResourceWithRawResponse",
    "AsyncExperiencesResourceWithRawResponse",
    "ExperiencesResourceWithStreamingResponse",
    "AsyncExperiencesResourceWithStreamingResponse",
]
